//----------------------------------------------------------------------------
// ReportLabel.cs
//
// This is a part of the C1FlexReportDesigner application sources.
// Copyright (C) GrapeCity Inc.
// All rights reserved.
//----------------------------------------------------------------------------
using System;
using System.Collections;
using System.Windows.Forms;

namespace FlexReportDesignerApp
{
	/// <summary>
	/// Summary description for ReportLabel.
	/// </summary>
	public class ReportLabel
	{
		// ** fields
		internal string _id, _desc;
		internal int _across, _wid, _hei, _xspc, _yspc;
		internal int _mleft, _mright, _mtop, _mleftlbl, _mtoplbl;
		internal bool _metric, _continuous, _portrait;

		// ** ctor

		public ReportLabel(string id, string desc, int across, bool metric,
			int wid, int hei, int xspc, int yspc, int mleft, int mright, int mtop, 
			int mleftlbl, int mtoplbl, bool continuous, bool portrait)
		{
			_id = id;
			_desc = desc;
			_across = across;
			_metric = metric;
			_wid = wid;
			_hei = hei;
			_xspc = xspc;
			_yspc = yspc;
			_mleft = mleft;
			_mright = mright;
			_mtop = mtop;
			_mleftlbl = mleftlbl;
			_mtoplbl = mtoplbl;
			_continuous = continuous;
			_portrait = portrait;
		}

		// ** object model

		static public void Fill(ListView list, bool metric, bool continuous)
		{
			list.Items.Clear();
			foreach (ReportLabel label in LabelArray)
			{
				if (label._continuous == continuous && label._metric == metric)
				{
					ListViewItem lvi = new ListViewItem(new string[] 
					{
						label._id,
						label._desc,
						label._across.ToString()
					});
					lvi.Tag = label;
					list.Items.Add(lvi); 
				}
			}
			list.Items[0].Selected = true;
		}

		static private ReportLabel[] _labelArray;
		static private ReportLabel[] LabelArray
		{
			get
			{
				if (_labelArray == null)
				{
					ArrayList al = new ArrayList();
					foreach (string str in _averyLabels)
					{
						string[] data = str.Split(',');
						al.Add(new ReportLabel(
							data[0],				// ID        label id
							data[1],				// DESC      description
							int.Parse(data[2]),		// ACROSS    labels across a page
							data[3] != "0",			// METRIC    metric/english units
							int.Parse(data[4]),		// WID       label width (all measurements in twips)
							int.Parse(data[5]),		// HEI       label height
							int.Parse(data[6]),		// XSPC      horz space between labels
							int.Parse(data[7]),		// YSPC      vert space between labels
							int.Parse(data[8]),		// MLEFT     page margin left
							int.Parse(data[9]),		// MRIGHT    page margin right
							int.Parse(data[10]),	// MTOP      page margin top
							int.Parse(data[11]),	// MLEFTLBL  label margin left
							int.Parse(data[12]),	// MTOPLBL   label margin top
							data[13] != "0",		// CONT      continuous/sheet
							data[14] != "0"			// PORTRAIT  portrait orientation
							));
					}
					_labelArray = (ReportLabel[])al.ToArray(typeof(ReportLabel));
				}
				return _labelArray;
			}
		}
		// -------------------------------------------------
		// Predefined label formats (Avery)
		// -------------------------------------------------
		// this is a comma-delimited file (no double-quotes)
		// with the following fields:
		// -------------------------------------------------
		// ID        label id
		// DESC      description
		// ACROSS    labels across a page
		// METRIC    metric/english units
		// WID       label width (all measurements in twips)
		// HEI       label height
		// XSPC      horz space between labels
		// YSPC      vert space between labels
		// MLEFT     page margin left
		// MRIGHT    page margin right
		// MTOP      page margin top
		// MLEFTLBL  label margin left
		// MTOPLBL   label margin top
		// CONT      continuous/sheet
		// PORTRAIT  portrait orientation
		// -------------------------------------------------
		//
		//ID,DESC,ACROSS,METRIC,WID,HEI,XSPC,YSPC,MLEFT,MRIGHT,MTOP,MLEFTLBL,MTOPLBL,CONT,PORTRAIT
		static string[] _averyLabels = new string[]
		{
			"5095,2 1/2\" x 3 3/8\",2,0,4860,3360,540,270,990,990,792,547,432,0,1",
			"5096,2 3/4\" x 2 3/4\",3,0,3456,3960,504,360,432,432,720,36,1656,0,1",
			"5097,1 1/2\" x 4\",2,0,5346,2160,684,0,432,432,1440,81,504,0,1",
			"5160,1\" x 2 5/8\",3,0,3366,1440,594,0,432,432,720,81,216,0,1",
			"5161,1\" x 4\",2,0,5256,1440,729,0,432,432,720,36,216,0,1",
			"5162,1 1/3\" x 4\",2,0,5346,1920,684,0,432,432,1170,81,216,0,1",
			"5163,2\" x 4\",2,0,5346,2880,684,0,432,432,720,81,288,0,1",
			"5164,3\" 1/3 x 4\",2,0,5616,4795,144,0,432,432,720,288,360,0,1",
			"5165,8 1/2\" x 11\",1,0,11376,14976,864,864,432,432,432,0,0,0,1",
			"5167,1/2\" x 1 3/4\",4,0,2376,720,576,0,432,432,720,216,72,0,1",
			"5196,2 3/4\" x 2 3/4\",3,0,3456,3960,504,360,432,432,720,36,288,0,1",
			"5197,1 1/2\" x 4\",2,0,5346,2160,684,0,432,432,1440,81,216,0,1",
			"5198,1 2/3\" x 3 1/2\",2,0,5040,2400,720,0,720,720,720,288,216,0,1",
			"5260,1\" x 2 5/8\",3,0,3366,1440,594,0,432,432,720,81,216,0,1",
			"5261,1\" x 4\",2,0,5346,1440,639,0,432,432,720,81,216,0,1",
			"5262,1 1/3\" x 4\",2,0,5346,1920,684,0,432,432,1170,81,216,0,1",
			"5263,2\" x 4\",2,0,5346,2880,684,0,432,432,720,81,288,0,1",
			"5266,2/3\" x 3 7/16\",2,0,4950,960,762,0,720,720,720,288,216,0,1",
			"5267,1/2\" x 1 3/4\",4,0,2376,720,576,0,432,432,720,216,72,0,1",
			"5361,2\" x 3 1/4\",1,0,4680,2880,0,2399,6059,1350,1200,288,288,0,1",
			"5362,2\" x 3 1/4\",1,0,4680,2880,0,2399,6059,1350,1200,288,288,0,1",
			"5364,2 1/16\" x 3 7/8\",1,0,5580,2970,0,2309,6120,450,1137,288,288,0,1",
			"5371,2\" x 3 1/2\",2,0,5040,2880,0,0,1080,1080,720,216,216,0,1",
			"5376,2\" x 3. 1/2\",2,0,5040,2880,0,0,1080,1080,720,216,216,0,1",
			"5377,2\" x 3 1/2\",2,0,5040,2880,0,0,1080,1080,720,216,216,0,1",
			"5383,2 1/6\" x 3 1/2\",2,0,5040,3120,0,0,1080,1080,1680,288,216,0,1",
			"5384,3\" x 4\",2,0,5616,4320,144,0,432,432,1440,216,288,0,1",
			"5385,2 1/6\" x 4\",2,0,5616,3150,144,0,432,432,1620,216,288,0,1",
			"5386,3\" x 5\",1,0,7200,4320,0,0,2520,2520,1440,360,360,0,1",
			"5388,3\" x 5\",1,0,7200,4320,0,0,2520,2520,1440,360,360,0,1",
			"5389,4\" x 6\",1,0,8640,5760,0,0,1800,1800,2100,360,360,0,1",
			"5395,2 1/2\" x 3 3/8\",2,0,4860,3360,540,270,990,990,792,360,360,0,1",
			"5660,1\" x 2 13/16\",3,0,3186,1440,864,0,432,432,720,0,216,0,1",
			"5661,1\" x 4 1/4\",2,0,5256,1440,864,0,432,432,720,0,216,0,1",
			"5662,1 1/3\" x  4 1/4\",2,0,5256,1920,864,0,432,432,1200,29,216,0,1",
			"5663,2\" x 4 1/4\",2,0,5256,2880,864,0,432,432,720,29,288,0,1",
			"5664,3 1/3\" x 4 1/4\",2,0,5256,4799,864,0,432,432,720,0,360,0,1",
			"5883,2 1/6\" x 3 1/2\",2,0,5040,3120,0,0,1080,1080,1680,547,504,0,1",
			"5895,2 1/3\" x 3 3/8\",2,0,4860,3330,540,270,990,990,810,547,504,0,1",
			"5896,2 3/4\" x 2 3/4\",3,0,3456,3960,504,360,432,432,720,36,1656,0,1",
			"5897,1 1/2\" x 4\",2,0,5346,2160,684,0,432,432,1440,81,504,0,1",
			"5920,1\" x 2 5/8\",3,0,3366,1440,594,0,432,432,720,81,216,0,1",
			"5922,1 1/3\" x 4\",2,0,5346,1920,684,0,432,432,1170,81,216,0,1",
			"5927,1/2\" x 1 3/4\",4,0,2376,720,576,0,432,432,720,216,72,0,1",
			"6460,1\" x 2 5/8\",3,0,3456,1440,504,0,432,432,720,126,216,0,1",
			"6464,3 1/3\" x 4\",2,0,5346,4800,684,0,432,432,720,153,360,0,1",
			"6466,2/3\" x 7/16\",2,0,4950,960,809,0,765,765,720,288,216,0,1",
			"6490,2\" x 2 11/16\",3,0,3366,2880,639,0,432,432,720,36,288,0,1",
			"8160,1\" x 2 5/8\",3,0,3456,1440,504,0,432,432,720,126,216,0,1",
			"8161,1\" x 4\",2,0,5346,1440,684,0,432,432,720,81,216,0,1",
			"8162,1 1/3\" x 4\",2,0,5346,1920,684,0,432,432,1200,81,216,0,1",
			"8163,2\" x 4\",2,0,5346,2556,684,324,432,432,432,81,126,0,1",
			"8164,3 1/3\" x 4\",2,0,5346,4800,684,0,432,432,720,153,360,0,1",
			"8165,11\" x 8 1/2\",1,0,11376,14976,864,864,432,432,432,0,0,0,1",
			"8166,2/3\" x 3 7/16\",2,0,4950,960,809,0,765,765,720,288,216,0,1",
			"8167,1/2\" x 1 3/4\",4,0,2466,720,504,0,432,432,720,261,72,0,1",
			"8196,2 3/4\" x 2 3/4\",3,0,3456,3960,504,360,432,432,720,36,288,0,1",
			"8371,2\" x 3 1/2\",2,0,5040,2880,0,0,1080,1080,720,216,216,0,1",
			"8660,1\" x 2 5/8\",3,0,3456,1440,504,0,432,432,720,126,216,0,1",
			"8662,1 1/3\" x 4\",2,0,5346,1920,684,0,432,432,1200,81,216,0,1",
			"8663,2\" x 4\",2,0,5346,2556,684,324,432,432,432,81,126,0,1",
			"2160 (1/2 sheet),1\" x 2 5/8\",1,0,3787,1440,0,0,1166,1166,720,288,180,0,1",
			"2162 (1/2 sheet),1 1/3\" x 4\",1,0,5242,1915,518,0,432,432,720,29,180,0,1",
			"2163 (1/2 sheet),2\" x 4\",1,0,5242,2880,518,0,432,432,720,29,180,0,1",
			"2180 (1/2 sheet),2/3\" x 3 7/16\",1,0,4954,950,0,0,583,583,720,288,180,0,1",
			"2181 (1/2 sheet),2/3\" x 3 7/16\",1,0,4954,950,0,0,583,583,720,288,180,0,1",
			"2186 (1/2 sheet),2\" x 2 3/4\",1,0,3960,2880,0,0,1080,1080,720,288,180,0,1",
			"2660 (1/2 sheet),1\" x 2 5/8\",1,0,3787,1440,0,0,1166,1166,720,288,180,0,1",
			"2662 (1/2 sheet),1 1/3\" x 4 1/4\",1,0,5256,1915,864,0,432,432,720,0,360,0,1",
			"2663 (1/2 sheet),2\" x 4 1/4\",1,0,5256,2880,864,0,432,432,720,0,360,0,1",
			"5199-F,1 5/6\" x 3 1/16\",2,0,4500,2640,360,0,1530,1530,1305,288,288,0,1",
			"5199-S,2/3\" x 5 13/16\",1,0,8352,960,0,0,1935,1935,720,288,72,0,1",
			"Index Maker - 3 Tab,1/2\" x 3 1/8\",2,0,4500,720,1800,0,720,720,720,288,72,0,1",
			"Index Maker - 5 Tab,1/2\" x 1 3/4\",4,0,2520,720,432,0,432,432,720,202,72,0,1",
			"Index Maker - 8 Tab,1/2\" x 1 1/8\",4,0,1620,720,1440,0,720,720,720,202,72,0,1",
			"Ready Index - 12,11/16\" x 3\",1,0,4536,1220,0,0,7272,432,540,216,360,0,1",
			"Ready Index - 15,0.63\" x 3\",1,0,4536,924,0,0,7272,432,1123,216,360,0,1",
			"Ready Index - 5,2\" x 3\",1,0,4536,2880,0,0,7272,432,619,216,360,0,1",
			"Ready Index - 8,1.22\" x 3\",1,0,4536,1755,0,0,7272,432,810,216,504,0,1",
			"Ready Index - 8c,1.22\" x 3\",1,0,4536,1755,0,0,7272,432,810,216,504,0,1",
			"4144,15/16\" x 2 1/2\",3,0,3600,1350,144,90,540,540,45,288,216,1,1",
			"4145,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4146,1 7/16\" x 4\",1,0,5760,2070,0,90,0,0,45,288,216,1,1",
			"4160,2 7/16\" x 3 1/2\",1,0,5040,3510,0,90,0,0,45,461,576,1,1",
			"4161,2 15/16\" x 4\",1,0,5760,4230,0,90,0,0,45,360,360,1,1",
			"4162,15/16\"x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4163,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4166,3\" x 5\",1,0,7200,4320,0,0,0,0,0,360,360,1,1",
			"4167,3 1/2\" x 6\",1,0,8640,5040,0,0,0,0,0,360,360,1,1",
			"4168,2 1/6\" x 4\",1,0,5760,3120,0,0,0,0,0,360,360,1,1",
			"4169,3\" x 5\",1,0,7200,4320,0,0,0,0,0,360,360,1,1",
			"4240,1 1/4\" x 4 3/4\",1,0,6840,1800,0,360,0,0,180,288,216,1,1",
			"4241,2 3/4\" x 2 3/4\",1,0,3960,3960,0,360,540,540,180,360,360,1,1",
			"4249,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4250,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4251,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4253,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4254,15/16\" x 3 1/2\",1,0,5040,1350,0,90,0,0,45,288,216,1,1",
			"4255,7/16\" x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,216,1,1",
			"4256,7/16\"x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,216,1,1",
			"4257,7/16\" x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,216,1,1",
			"4258,7/16\" x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,216,1,1",
			"4259,7/16\"x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,216,1,1",
			"4266,7/16\" x 3 1/2\",1,0,5040,630,0,90,0,0,45,202,144,1,1",
			"J8160,38.10 mm x 63.50 mm,3,1,3456,2160,269,0,432,432,850,268,170,0,1",
			"J8162,33.87 mm x 99.06 mm,2,1,5262,1915,467,0,432,432,734,248,170,0,1",
			"J8163,38.10 mm x 99.06 mm,2,1,5262,2160,467,0,432,432,850,248,255,0,1",
			"J8165,67.73 mm x 99.06 mm,2,1,5262,3840,467,0,432,432,720,248,283,0,1",
			"J8166,52 mm x 72 mm,2,1,4082,2948,1208,0,1321,1321,1090,288,288,0,1",
			"J8360,38.10 mm x 63.50 mm,3,1,3456,2160,269,0,432,432,850,268,170,0,1",
			"J8362,33.87 mm x 99.06 mm,2,1,5262,1915,467,0,432,432,734,248,170,0,1",
			"J8363,38.10 mm x 99.06 mm,2,1,5262,2160,467,0,432,432,850,248,255,0,1",
			"J8365,67.73 mm x 99.06 mm,2,1,5262,3840,467,0,432,432,720,248,283,0,1",
			"J8560,38.10 mm x 63.50 mm,3,1,3456,2160,269,0,432,432,850,268,170,0,1",
			"J8562,33.87 mm x 99.06 mm,2,1,5262,1915,467,0,432,432,734,248,170,0,1",
			"J8563,38.10 mm x 99.06 mm,2,1,5262,2160,467,0,432,432,850,248,255,0,1",
			"J8566,52 mm x 72 mm,2,1,4082,2948,1208,0,1321,1321,1090,288,288,0,1",
			"L2160 (1/2 sheet),25.4 mm x 66.68 mm,1,1,3787,1440,0,0,1080,1080,720,288,288,0,1",
			"L2162 (1/2 sheet),33.87 mm x 101.6 mm,1,1,5242,1915,518,0,432,432,720,29,180,0,1",
			"L2163 (1/2 sheet),50.8 mm x 101.6 mm,1,1,5242,2880,518,0,432,432,720,29,180,0,1",
			"L2164 (1/2 sheet),14 mm x 50 mm,2,1,2317,794,608,0,432,432,720,29,180,0,1",
			"L2186 (1/2 sheet),50.8 mm x 69.85 mm,1,1,3960,2880,0,0,1080,1080,720,288,180,0,1",
			"L7159,33.46 mm x 64 mm,3,1,3484,1920,288,0,432,432,690,216,288,0,1",
			"L7160,38.10 mm x 63.50 mm,3,1,3456,2160,269,0,432,432,850,268,170,0,1",
			"L7161,46.56 mm x 63.50 mm,3,1,3456,2636,269,0,432,432,498,268,170,0,1",
			"L7162,33.87 mm x 99.06 mm,2,1,5262,1915,467,0,432,432,734,248,170,0,1",
			"L7163,38.10 mm x 99.06 mm,2,1,5262,2160,467,0,432,432,850,248,255,0,1",
			"L7164,71.97 mm x 63.50 mm,3,1,3530,3729,183,346,432,432,432,390,110,0,1",
			"L7165,67.73 mm x 99.06 mm,2,1,5262,3840,467,0,432,432,720,248,283,0,1",
			"L7166,93.13 mm x 99.06 mm,2,1,5262,5280,467,0,432,432,493,248,283,0,1",
			"L7167,289.05 mm x 199.60 m,1,1,10985,15352,298,864,432,432,432,418,135,0,1",
			"L7168,144.5 mm x 199.60 mm,1,1,10929,7244,354,864,432,432,432,390,135,0,1",
			"L7169,99.1 mm x 139 mm,2,1,7869,5278,0,484,540,540,432,288,118,0,0",
			"L7173,57 mm x 99.06 mm,2,1,5282,2944,421,288,432,432,432,121,144,0,1",
			"L7413,50.8 mm x 90 mm,2,1,5102,2880,0,0,850,850,1200,288,288,0,1",
			"L7414/15,52 mm x 90 mm,2,1,5102,2948,0,0,850,850,1000,288,288,0,1",
			"L7417/19,54 mm x 90 mm,2,1,5102,3061,0,0,850,850,765,288,288,0,1",
			"L7418,55 mm x 86 mm,2,1,4876,3118,0,0,1077,1077,2183,288,288,0,1",
			"L7421,97.29 mm x 139.37 mm,2,1,7901,5516,0,0,537,537,432,288,288,0,0",
			"L7562,33.87 mm x 99.06 mm,2,1,5262,1915,467,0,432,432,734,248,170,0,1",
			"L7563,38.10 mm x 99.06 mm,2,1,5262,2160,467,0,432,432,850,248,170,0,1",
			"L7565,67.73 mm x 99.06mm,2,1,5262,3840,467,0,432,432,720,248,283,0,1",
			"L7630,63.50 mm x 63.50 mm,3,1,3304,3600,551,255,432,432,708,532,680,0,1",
			"L7651,21.17 mm x 38.10 mm,5,1,1806,1196,495,0,432,432,617,0,113,0,1",
			"L7654-F,46.56 mm x 76.20 mm,2,1,4320,2636,141,0,1531,1531,498,425,170,0,1",
			"L7654-S,16.93 mm x 144.78 mm,1,1,8208,960,0,0,1814,1814,720,425,113,0,1",
			"L7663,33.76 mm x 119.38 mm,1,1,6768,1913,0,0,2551,2551,762,425,227,0,1",
			"L7664,71.97 mm x 70.00 mm,2,1,3969,3729,1928,346,992,992,432,425,110,0,1",
			"L7666,52 mm x 72 mm,2,1,4082,2948,1208,0,1321,1321,1090,288,288,0,1",
			"L7671,46.56 mm x 76.10 mm,2,1,4320,2636,141,0,1531,1531,498,425,170,0,1",
			"L7674,16.93 mm x 144.78 mm,1,1,8208,960,0,0,1814,1814,720,425,113,0,1",
			"EXPE 120.7,74 mm x 120 mm,1,1,6804,4321,0,0,567,567,62,340,113,1,1",
			"FAB 03,36.1 mm x 69 mm,1,1,4003,2189,91,0,675,675,71,340,113,1,1",
			"OML 101,24 mm x 102 mm,1,1,5783,1503,0,0,709,709,71,340,113,1,1",
			"OML 102,37 mm x 102 mm,1,1,5783,2240,0,0,709,709,71,340,113,1,1",
			"OML 103,49 mm x 102 mm,1,1,5783,2920,0,0,709,709,71,340,113,1,1",
			"OML 105,49 mm x 127 mm,1,1,7201,2920,0,0,709,709,71,340,113,1,1",
			"OML 202,37 mm x 102 mm,2,1,5874,2240,91,0,675,675,71,340,113,1,1",
			"OML 203,49 mm x 102 mm,2,1,5874,2920,91,0,675,675,71,340,113,1,1",
			"Tab1 102.3,36.1 mm x 102 mm,1,1,5783,2047,0,0,709,709,57,340,113,1,1",
			"Tab1 107.2,23.4 mm x 107 mm,1,1,6067,1440,0,0,567,567,57,340,113,1,1",
			"Tab1 107.3,36.1 mm x 107 mm,1,1,6067,2160,0,0,567,567,57,340,113,1,1",
			"Tab1 107.4,48.8 mm x 107 mm,1,1,6067,2880,0,0,567,567,57,340,113,1,1",
			"Tab1 125.7,74 mm x 125 mm,1,1,7088,4321,0,0,567,567,62,340,113,1,1",
			"Tab1 85.10,108 mm x 85 mm,1,1,4820,7201,0,0,765,765,539,340,113,1,1",
			"Tab1 89.23,23.4 mm x 89 mm,1,1,5046,1440,0,0,709,709,57,340,113,1,1",
			"Tab1 89.36,36.1 mm x 89 mm,1,1,5046,2160,0,0,709,709,57,340,113,1,1",
			"Tab2 107.2,23.4 mm x 107 mm,2,1,6350,1440,284,0,454,454,57,340,113,1,1",
			"Tab2 107.3,36.1 mm x 107 mm,2,1,6350,2160,284,0,454,454,57,340,113,1,1",
			"Tab2 89.23,23.4 mm x 89 mm,2,1,5330,1440,284,0,595,595,57,340,113,1,1",
			"Tab2 89.36,36.1 mm x 89 mm,2,1,5330,2160,284,0,595,595,57,340,113,1,1"
		};
	}
}
